#!/usr/bin/perl -w

# Author   : Andreas Hirczy <ahi@itp.tugraz.at>
# available: http://itp.tugraz.at/~ahi/Computer/SW/ganglia_YYYY.MM.DD.tar.gz
# License  : GNU General Public License
#
#    This program is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 2 of the License, or (at your
#    option) any later version.
#
#    This program is distributed in the hope that it will be useful, but
#    WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Description : report hard disk health status - temperature
# Language    : Perl
# Category    : Health Monitoring::Hard Disk
# Dependencies: hddtemp, perl
# Target OS   : Linux (might work on others)
#
# Reports hard disk health status: currently only disc temperature obtained
# through "hddtemp" - reports one temperature per drive (disc_temp_XXX) found
# and overall maximum temperature ("disc_temp_max").
#
# Might be extended to use data obtained via "smartctl"
# <http://smartmontools.sourceforge.net/> if I find a set of sensible values
# to use: Raw_Read_Error_Rate, Reallocated_Sector_Ct, Power_On_Hours, ...

my $HDDTEMP = "/usr/sbin/hddtemp";
my $GMETRIC = "/usr/bin/gmetric";
my %GM_PARA = ("--units=Celsius", "--type=int16");
my $HDD     = "/dev/sd? /dev/hd?";
my $LIMIT_U = 45;

my $TEMP_MAX = -274;


open (HDD_TEMP, "$HDDTEMP $HDD 2>/dev/null|")
  or die "cannot spawn '$HDDTEMP $HDD': $?";

while (<HDD_TEMP>) {
  chomp;
  next unless /^\/dev\/(.d.):.*: (\d*) C/;
  # print "$1: $2\n";
  system ($GMETRIC, "--name=disc_temp_$1", "--value=$2", %GM_PARA);
  if ($LIMIT_U <= $2) {
    print "Temperature on HardDisc $1 is $2 -- over limit ($LIMIT_U)\n";
  }
  if ($TEMP_MAX < $2) {
    $TEMP_MAX = $2;
  }
}
close HDD_TEMP;

if ($TEMP_MAX != -274) {
  # print "MAX: $TEMP_MAX\n";
  system ($GMETRIC, "--name=disc_temp_max", "--value=$TEMP_MAX", %GM_PARA);
}
